import { Socket } from 'net';

export interface IHapInstance {
    displayName: string
    name: string;
    ipAddress: string;
    port: number;
    username: string;
}

export interface IHapEvInstance {
    name: string;
    ipAddress: string;
    port: number;
    username: string;
    evCharacteristics?: { aid: number, iid: number, ev: boolean }[];
    socket?: Socket;
}

export interface IInstanceResp {
    ipAddress: string;
    port: number;
    username: string;
    name: string;
}

export interface ICharacteristicResp {
    iid: number;
    type: string;
    description: string;
    value: number | string | boolean;
    format: 'bool' | 'int' | 'float' | 'string' | 'uint8' | 'uint16' | 'uint32' | 'uint64' | 'data' | 'tlv8' | 'array' | 'dictionary';
    perms: Array<'pr' | 'pw' | 'ev' | 'aa' | 'tw' | 'hd'>;
    unit?: 'unit' | 'percentage' | 'celsius' | 'arcdegrees' | 'lux' | 'seconds';
    maxValue?: number;
    minValue?: number;
    minStep?: number;
}

export interface IServiceResp {
    iid: number;
    type: string;
    primary: boolean;
    hidden: boolean;
    linked?: Array<number>;
    characteristics: Array<ICharacteristicResp>;
}

export interface IAccessoryResp {
    instance: IInstanceResp
    aid: number;
    services: Array<IServiceResp>;
}

export interface IHapAccessoriesRespType {
    accessories: Array<IAccessoryResp>;
}

export interface IServiceType {
    aid: number;
    iid: number;
    uuid: string;
    type: string;
    linked?: Array<number>;
    linkedServices?: {
        [iid: number]: IServiceType;
    };
    hidden?: boolean;
    humanType: string;
    serviceName: string;
    serviceCharacteristics: ICharacteristicType[];
    accessoryInformation: any;
    refreshCharacteristics?: () => Promise<IServiceType>;
    setCharacteristic?: (iid: number, value: number | string | boolean) => Promise<IServiceType | ICharacteristicType>;
    getCharacteristic?: (type: string) => ICharacteristicType;
    values: any;
    instance: {
        ipAddress: string;
        port: number;
        username: string;
        name: string;
    };
    uniqueId?: string;
}

export interface ICharacteristicType {
    aid: number;
    iid: number;
    uuid: string;
    type: string;
    serviceType: string;
    serviceName: string;
    description: string;
    value: number | string | boolean;
    format: 'bool' | 'int' | 'float' | 'string' | 'uint8' | 'uint16' | 'uint32' | 'uint64' | 'data' | 'tlv8' | 'array' | 'dictionary';
    perms: Array<'pr' | 'pw' | 'ev' | 'aa' | 'tw' | 'hd'>;
    unit?: 'unit' | 'percentage' | 'celsius' | 'arcdegrees' | 'lux' | 'seconds';
    maxValue?: number;
    minValue?: number;
    minStep?: number;
    canRead: boolean;
    canWrite: boolean;
    ev: boolean;
    setValue?: (value: number | string | boolean) => Promise<ICharacteristicType>;
    getValue?: () => Promise<ICharacteristicType>;
}

export const createDefaultCharacteristicType = (): ICharacteristicType => {
    return {
        aid: -1,
        iid: -1,
        uuid: "",
        type: "",
        serviceType: "",
        serviceName: "",
        description: "",
        value: "",
        format: 'bool',
        perms: [],
        canRead: false,
        canWrite: false,
        ev: false,
    };
}
